package de.fitko.fitconnect.client;

import de.fitko.fitconnect.api.domain.auth.OAuthToken;
import de.fitko.fitconnect.api.domain.model.metadata.Metadata;
import de.fitko.fitconnect.api.domain.model.metadata.attachment.Attachment;
import de.fitko.fitconnect.api.domain.model.submission.SubmissionRequest;
import de.fitko.fitconnect.api.services.Subscriber;

import java.util.Collections;
import java.util.List;
import java.util.Optional;

/**
 * A fluent client for handing in a subscription for a {@link Subscriber}
 */
public class SubscriberClient extends Client {

    private SubscriberClient() {}

     public static Authenticate builder() {
         final Subscriber subscriber = getService(Subscriber.class);
         return new ClientBuilder(subscriber);
    }

    public interface Authenticate {
        PollSubmissions authenticate(String clientId, String secret, String... scope);
    }

    public interface PollSubmissions {

        List<SubmissionRequest> getAvailableSubmissions(String destinationId);
        RequestMetadata requestSubmission(SubmissionRequest submission);
    }

    public interface RequestMetadata {
        RequestAttachments requestMetadata();
    }

    public interface  RequestAttachments {
        ConfirmSubmission requestAttachments();
    }

    public interface ConfirmSubmission {
        AccessData confirmSubmission();
    }

    public interface AccessData {
        SubmissionRequest getSubmission();
        List<Attachment> getAttachments();
        Metadata getMetadata();
    }


    public static class ClientBuilder implements Authenticate, PollSubmissions, RequestMetadata, RequestAttachments, ConfirmSubmission, AccessData {

        private final Subscriber subscriber;

        private Optional<SubmissionRequest> submission = Optional.empty();
        private Optional<Metadata> metadata = Optional.empty();
        private List<Attachment> attachments = Collections.emptyList();
        private Optional<OAuthToken> token;

        private ClientBuilder(Subscriber subscriber) {
            this.subscriber = subscriber;
        }

        @Override
        public PollSubmissions authenticate(String clientId, String secret, String... scope) {
            this.token = subscriber.retrieveOAuthToken(clientId, secret, scope);
            return this;
        }

        @Override
        public List<SubmissionRequest> getAvailableSubmissions(String destinationId) {
            throw new UnsupportedOperationException("not yet implemented");
        }

        @Override
        public RequestMetadata requestSubmission(SubmissionRequest submission) {
            throw new UnsupportedOperationException("not yet implemented");
        }


        @Override
        public RequestAttachments requestMetadata() {
            throw new UnsupportedOperationException("not yet implemented");
        }

        @Override
        public ConfirmSubmission requestAttachments() {
            throw new UnsupportedOperationException("not yet implemented");
        }

        @Override
        public AccessData confirmSubmission() {
            throw new UnsupportedOperationException("not yet implemented");
        }

        @Override
        public SubmissionRequest getSubmission() {
            return this.submission.orElseThrow();
        }

        @Override
        public List<Attachment> getAttachments() {
            return this.attachments;
        }

        @Override
        public Metadata getMetadata() {
            return this.metadata.orElseThrow();
        }
    }
}